@@ -829,6 +829,8 @@ Look here: https://github.com/gnudist/littleorm/tree/master/examples
 =head1 AUTHOR
 
 Eugene Kuzin, C<< <eugenek at 45-98.org> >>, JID: C<< <gnudist at jabber.ru> >>
+with significant contributions by
+Kain Winterheart, C<< <kain.winterheart at gmail.com> >>
 
 =head1 BUGS
 
@@ -854,6 +856,11 @@ You can also look for information at:
 
 =over 4
 
+
+=item * Project home:
+
+L<https://github.com/gnudist/littleorm>
+
 =item * RT: CPAN's request tracker (report bugs here)
 
 L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=LittleORM>
@@ -919,4 +926,4 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =cut
 
-1; # End of LittleORM
+42; # End of LittleORM
